/* code used for the FOGA 05 and TCS 05 papers */

#include<std.h>
#include<codeMap.h>
#include<list.h>
#include<algos.h>
#include<testFunctions.h>
#include<distributions.h>
#include<search.h>

struct Evaluation{
  bool newProblem;
  uint length,cost,Bcost;
  enum{ none, CountingOnes, HXOR, MAXSAT, trap } problem;
  TestFunctions::MAXSAT *ms;

  /*  struct hashFunction{
    size_t operator()(const intA& x) const{ 
      int k=0;
      for(uint i=0;i<15 && i<x.N;i++) k|=x(i)<<i;
      return k;
    }
    bool operator()(const intA& a,const intA& b) const{ return a==b; }
  };
  hash_set<intA,hashFunction,hashFunction> evaluated;*/

  Evaluation(){ reset(); problem=none; ms=0; }
  ~Evaluation(){
    if(ms) delete ms;
  }

  void reset(){
    TestFunctions::kTrapMix.resize(0);
    newProblem=true;
    cost=Bcost=0;
  }

  double operator()(const intA& x){
    static int msNum=0;
    byteA y;
    intA X; (intA&)X=x;

    //check the hash set first
    if(newProblem){
      //evaluated.clear();
    }else{
      /*      hash_set<intA,hashFunction,hashFunction>::iterator i;
      i=evaluated.find(X);
      if(i!=evaluated.end()){
	X.w=((intA&)(*i)).w;
	return X.w;
      }
      */
    }

    cost++;
    Bcost+=x.N;
    switch(problem){
    case CountingOnes:
      return TestFunctions::CountingOnes(x,length);
    case HXOR:
      return TestFunctions::HXOR(x,length);
    case trap:
      return TestFunctions::kTrap(x,4,length);
    case MAXSAT:
      if(!ms) ms=new TestFunctions::MAXSAT();
      if(newProblem){
	msNum++;
	msNum=2;
	ms->readCNF(STRING("../../../libs/SATLIB/uf"<<length<<"/uf"<<length<<"-0"<<msNum<<".cnf"));
	std::cout <<"opening " <<msNum <<"th MAXSAT problem" <<std::endl;
	newProblem=false;
      }
      y.copy(x);
      if(y.N>length) y.resizeCopy(length);
      return (*ms)(y);
    default: HALT("unknown problem");
    }
    //evaluated.insert(X);
    HALT("shouldn't be here!");
    return -1.;
  }
};


void paperTCS_GA(){
  Search<Evaluation,PairCodeGA2,intAL> search;

  search.E.problem=Evaluation::HXOR;
  search.hillClimb=false;

  system(STRING("mkdir TCS-GA"));
  std::ofstream sout("z-stat");
  uint k,S;
  for(k=1;k<20;k++){
    S=1<<k;
    search.maxLength=S;
    search.E.length=S;
    sout <<S <<' '; 
    search.runs(20,&sout);
    sout <<std::endl;
    system(STRING("mv z-evo TCS-GA/evo-"<<S));
    system(STRING("mv z-run TCS-GA/run-"<<S));
    system(STRING("cp -f z-stat TCS-GA/stat"));
  }
}

void paperTCS_EDA(){
  Search<Evaluation,PairCodeEDA,intAL> search;

  search.E.problem=Evaluation::HXOR;
  search.H.initL=2;

  /*search.H.usePool=true;
  search.H.lambda=1000;
  search.H.poolSize=100;
  search.H.selRate=1.;*/

  system(STRING("mkdir TCS-EDA"));
  std::ofstream sout("z-stat");
  uint k,S;
  for(k=1;k<20;k++){
    S=1<<k;
    search.maxLength=S;
    search.E.length=S;
    sout <<S <<' '; 
    search.runs(20,&sout);
    sout <<std::endl;
    system(STRING("mv z-evo TCS-EDA/evo-"<<S));
    system(STRING("mv z-run TCS-EDA/run-"<<S));
    system(STRING("cp -f z-stat TCS-EDA/z-stat"));
  }
}

void paperTCS_EDA_trap(){
  Search<Evaluation,PairCodeEDA,intAL> search;

  search.E.problem=Evaluation::trap;
  search.H.beta=2.;
  search.H.varLen=false;

  system(STRING("mkdir TCS-EDA"));
  std::ofstream sout("z-stat");
  uint k,S;
  for(k=2;k<20;k++){
    S=1<<k;
    search.maxLength=S;
    search.E.length=S;
    sout <<S <<' '; 
    search.H.initL=S;
    search.runs(20,&sout);
    sout <<std::endl;
    system(STRING("mv z-evo TCS-EDA/evo-"<<S));
    system(STRING("mv z-run TCS-EDA/run-"<<S));
    system(STRING("cp -f z-stat TCS-EDA/z-stat"));
  }
}

int main(int argc, char* argv[]){
  MT::init(argc,argv);

  //paperTCS_GA();
  paperTCS_EDA();
  //paperTCS_EDA_trap();

  return 0;
}
