#ifndef MT_graph_h
#define MT_graph_h

//#define MT_GRAPHCHECK

#include "std.h"
#include "array.h"
#include <list>
#include <algorithm>
#ifdef MT_QT
#  include <GL/gl.h>
extern void glDrawRect(float x1,float y1,float z1,float x2,float y2,float z2,
		       float x3,float y3,float z3,float x4,float y4,float z4,
		       float r,float g,float b);
#endif
#define INSbefore 1
#define INSafter 0

// iterations:
// the _save versions allow for deletions during recursion,
// for_nodes(n) is the same as forall(n,*this)

#ifndef forNodes
#  define forNodes(n,G)        for(n=G.first;n;n=n->next)
#  define forNodes_rev(n,G)    for(n=G.last;n;n=n->prev)
#  define forNodes_save(n,m,G) for(n=G.first,m=n?n->next:0; n; n=m,m=n?n->next:0)
#endif
#ifndef forEdges
#  define forEdges(i,e,G)      for(i=0;i<G.E && ((e=G.edges(i)) || true);i++)
#  define forEdges_rev(i,e,G)  for(i=G.E;i-- && ((e=G.edges(i)) || true);)
#endif

#ifndef for_nodes
#  define for_nodes(n) for(n=first;n;n=n->next)
#  define for_nodes_rev(n) for(n=last;n;n=n->prev)
#  define for_nodes_save(n,sn) for(n=first,sn=n?n->next:0; n; n=sn,sn=n?n->next:0)
#endif

#define for_edges(i,e) for(i=0;i<E && ((e=edges(i)) || true);i++)
#define for_in_edges(e,n) for(e=n->firstIn;e;e=e->nextIn)
#define for_in_edges_rev(e,n) for(e=n->lastIn;e;e=e->prevIn)
#define for_in_edges_save(e,es,n) for(e=n->firstIn,es=e?e->nextIn:0;e;e=es,es=e?e->nextIn:0)
#define for_out_edges(e,n) for(e=n->firstOut;e;e=e->nextOut)
#define for_out_edges_save(e,es,n) for(e=n->firstOut,es=e?e->nextOut:0;e;e=es,es=e?e->nextOut:0)


struct GraphStdOptionals{
  static void glDrawEdge(void *e,float x0,float y0,float z0,float x1,float y1,float z1){}
  static void glDrawNode(void *n,float x,float y,float z){}
};

/*! some features that are associated to each node for graph
  display -- you have free access */
struct NodeDisp{
  float x,y,z,c,fx,fy,size; //coordinates, dummy, force, size
  byte shape; //!< shape type (for future use)
  Color col;  //!< display color
  bool hide;  //!< "do not display" flag
  NodeDisp(){
    x=y=z=0.; size=.01; shape=0; col.setRgb(.5,.5,.5); hide=false; }
  friend inline std::istream& operator>>(std::istream& is,NodeDisp& f){
    return is >>f.x >>f.y >>f.z >>f.col; }
  friend inline std::ostream& operator<<(std::ostream& os,const NodeDisp& f){
    return os <<f.x <<" "<<f.y <<" "<<f.z <<" "<<f.col; }
};
/*! some features that are associated to each edge for graph
  display -- you have free access */
struct EdgeDisp{
  float c;
  Color col; //!< display color
  bool hide; //!< "do not display" flag
  EdgeDisp(){ col.setRgb(1.,.0,.0); hide=false; }
};

/*! A general graph object. Edges and nodes can be of arbitrary struct or class type.

Please see the example \ref examples/graph/main.cpp .

A #node_map# allows to associate additional data to each node
(e.g., an extra sort index or a pointer to a twin node in another
graph).

I briefly describe the internal data structure
of the graph object which is fully accessible and provides easy
and flexible handling.

The graph is build up of two types of objects which have type
#Node# and #Edge#. A #Node# object is allocated for
each node of the graph and refered to by a pointer of type
#node#. Each #Node# object contains the user data: it is
derived from the template class NodeData. Thus simple dereference
of a #node# pointer allows quick access to the user data
associated with a node. Additionally it contains the following:
\code
Node *next,*prev;
Edge *firstIn,*lastIn,*firstOut,*lastOut;
uint Nin,Nout;
uint index;
NodeDisp *d;
\endcode
which are, in given order, the two pointers to the next and the
previous node in the node list; four
pointers to edges, namely the first and last input and output
edges; the number of input and output nodes; the index of this
node in the Graph::nodes array; and a pointer to a struct with display parameters, which is allocated when needed.

An #Edge# object is allocated for each edge of the graph and
refered to by a pointer of type #edge#. The type is derived from
the template class #EdgeData# - thus simple dereference of an
#edge# pointer allows quick access to the user data associated to
each edge. Additionally it contains the following:
\code
Node *from,*to;
Edge *nextOut,*prevOut,*nextIn,*prevIn;
uint index;
EdgeDisp *d;
\endcode
which are, in given order, the input and output nodes of this
edge; the next and previous edge in from's list of output edges
and the next and previous edge in to's list of input edges; the index of this edge in the Graph::edges array; and a pointer to a struct with display parameters.

*/
template<class NodeData,class EdgeData,class Optionals=GraphStdOptionals>
class Graph{
public: //types
  class Edge;
  typedef Graph<NodeData,EdgeData,Optionals> GraphT;

  class Node:public NodeData{
  public:
    Node *next,*prev;
    Edge *firstIn,*lastIn,*firstOut,*lastOut;
    uint Nin,Nout;
    uint index; //gives the node's index in the graph's #nodes# array
    NodeDisp *d;
  
    Node():NodeData(){ setNull(); }
    Node(const NodeData& n):NodeData(n){ setNull(); }
    ~Node(){ if(d) delete d; }
    NodeData& data(){ return (NodeData&)(*this); }
    NodeDisp& disp(){ if(!d) d=new NodeDisp; return *d; }
  private:
    void setNull(){ next=prev=0; firstIn=lastIn=firstOut=lastOut=0; Nin=Nout=index=0; d=0; }
  };
  /*! A node is a pointer to a #Node# object which is derived
    from your data type #NodeData#; see Graph class description. */ 
  typedef Node *node; //the iterator and pointer

  class Edge:public EdgeData{
  public:
    Node *from,*to;
    Edge *nextOut,*prevOut,*nextIn,*prevIn;
    uint index;
    EdgeDisp *d;

    Edge():EdgeData(){ setNull(); }
    Edge(const EdgeData& e):EdgeData(e){ setNull(); }
    ~Edge(){ if(d) delete d; }
    EdgeData& data(){ return (EdgeData&)(*this); }
    EdgeDisp& disp(){ if(!d) d=new EdgeDisp; return *d; }
  private:
    void setNull(){ from=to=0; nextOut=prevOut=nextIn=prevIn=0; index=0; d=0; }
  };
  /*! An edge is a pointer to a #Node# object which is derived
    from your data tyle #EdgeData# but additionally contains; see
    Graph class description. */
  typedef Edge *edge;

  /*! this class type realizes a mapping from each node of the graph
    to a datum of type #T# */
  template<class T>
  class node_map{
  public:
    MT::Array<T> A;
    node_map(){}
    node_map(const GraphT& G){ init(G); }
    void init(const GraphT& G){ A.resizeCopy(G.N); }
    T& operator[](node n){ return A(n->index); }
    const T& operator[](node n) const{ return A(n->index); }
  };

  /*! this class type realizes a mapping from each node of the graph
    to a datum of type #T# */
  template<class T>
  class edge_map{
  public:
    MT::Array<T> A;
    edge_map(){}
    edge_map(const GraphT& G){ init(G); }
    void init(const GraphT& G){ A.resizeCopy(G.E); }
    T& operator[](edge n){ return A(n->index); }
    const T& operator[](edge n) const{ return A(n->index); }
  };


public:
  uint N; //!< number of nodes
  uint E; //!< number of edges
  /*! the first node of the graph */
  node first;
  /*! the last node of the graph */
  node last;

  /*! an array of all nodes (pointers to Nodees): Actually all
    nodes are linked in a list via their prev and next pointers -- in
    principle this would be enough to keep track of them. Still, this
    array additionally comprises pointers to all nodes. (Resize is
    quick because of the MT::Array::flexiMem and MT::Array::memMove
    flags turned on.) This allows faster random access and quick
    checking if a node belongs to this graph. Note though that the
    ordering defined by the linked list of nodes, and the indexing
    defined by this array, generally don't coincide. Use
    indexAsOrdered() or orderAsIndexed() to synchronize the ordering
    and indexing.*/
  MT::Array<node> nodes;
  /*! an array of all edges (pointers to Edgees): see
    Graph::nodes for an explanation; use indexAllAsOrdered() to
    synchronize the ordering, defined by the edge lists, and the
    indexing, defined by this array */
  MT::Array<edge> edges;

  /*! tells whether the graph was defragmented (see defragment()) */
  bool defragmented;

  bool drawText; //!< option: write text of each node and edge
  //bool hideGraphPlug; 
  int graphLayout; //!< choose the graph layout

public:
  Graph(){
    N=E=0;
    first=last=0;
    nodes.memMove=true;
    edges.memMove=true;
    nodes.resize(0);
    edges.resize(0);
    defragmented=false;
    drawText=true;
    graphLayout=0;
  }
  ~Graph(){
    clear();
  }


public://{ basic access:
  /*! the i-th node */
  node operator()(uint i){ return nodes(i); }
  /*! the i-th edge */
  edge operator[](uint i){ return edges(i); }
  /*! a randomly (uniformly) chosen node */
  node rndNode(){ if(!N) return 0; return nodes(rnd(N)); }
  /*! a randomly (uniformly) chosen edge */
  edge rndEdge(){ if(!E) return 0; return edges(rnd(E)); }
  /*! return the connecting edge or 0 */
  edge getEdge(node from,node to) const{
    CHECK(checkNode(from) && checkNode(to),"getEdge: are not nodes");
    edge e;
    for_out_edges(e,from) if(e->to==to) return e;
    return 0;
  }
  /*! return true if n is a node of this graph */
  bool checkNode(node n) const{ return (n && n->index<N && nodes(n->index)==n); }
  /*! return true if e is a node of this graph */
  bool checkEdge(edge e) const{ return (e && e->index<E && edges(e->index)==e); }


public://{ the core methods: new and delete nodes and edges
  /*! inserts a new node after or before #where# (default: at the end
    of the node list); opt has to be INSafter or INSbefore,
    respectively */
  node new_node(node where=0,int opt=INSbefore,const NodeData* x=0){
    CHECK(opt==INSafter || opt==INSbefore,"Graph::new_node");
    CHECK(!defragmented,"new_node: can't mutate defragmented graph!");
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
    if(!where){
      if(opt==INSbefore) where=last; else where=0;
    }else{
      CHECK(checkNode(where),"new_node: where is not a node");
      if(opt==INSbefore) where=where->prev;
    }
    node n;
    if(!x) n=new Node;
    else   n=new Node(*x);
    if(!n) HALT("MT::Graph failed node memory allocation (graph size="<<N*sizeof(Node)+E*sizeof(Edge)<<"bytes)");
    //link up in list:
    if(!N){
      n->prev=0;      n->next=0;
      first=n;	      last=n;
    }else{
      if(where){ n->prev=where; n->next=where->next; }else{ n->prev=0; n->next=first; }
      if(n->prev) n->prev->next=n; else first=n;
      if(n->next) n->next->prev=n; else last=n;
    }
    //register in table:
    n->index=N;
    N++;
    nodes.resizeCopy(nodes.N+1);
    nodes(n->index)=n;
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
    return n;
  }

  /*! appends a new node and initialize it by #x# */
  node new_node(const NodeData& x){ return new_node(last,INSafter,&x); }

  /*! deletes the node and all attached edges. #n# becomes invalid! */
  void del_node(node n){
    CHECK(checkNode(n),"del_node: is not a node");
    CHECK(!defragmented,"del_node: can't mutate defragmented graph!");
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
    if(!n) return;
    //unlink from list:
    while(n->firstIn) del_edge(n->firstIn);
    while(n->firstOut) del_edge(n->firstOut);
    if(n==first) first=n->next; else n->prev->next=n->next;
    if(n==last)  last =n->prev; else n->next->prev=n->prev;
    //delete from table:
    nodes(n->index)=nodes(N-1);
    nodes(n->index)->index=n->index;
    N--;
    nodes.resizeCopy(nodes.N-1);
    delete n;
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
  }

  //! delete a whole set of nodes
  void del_nodes(const std::list<node>& set){
    typename std::list<node>::const_iterator n;
    for(n=set.begin();n!=set.end();n++) del_node_const(*n);
  }

  /*! appends a new edge */
  edge new_edge(node from,node to,const EdgeData* x=0){
    CHECK(checkNode(from) && checkNode(to),"new_edge: are not nodes");
    CHECK(!defragmented,"new_edge: can't mutate defragmented graph!");
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
    edge e;
    if(!x) e=new Edge; else e=new Edge(*x);
    if(!e) HALT("MT::Graph failed edge memory allocation (graph size="<<N*sizeof(Node)+E*sizeof(Edge)<<"bytes)");
    //register in table:
    e->index=E;
    E++;
    edges.resizeCopy(edges.N+1);
    edges(e->index)=e;
    connectToNodes(e,from,to);
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
    return e;
  }

  /*! deletes the edge -- ensures save for-loops in the same way as
    del_node */
  void del_edge(edge e){
    CHECK(checkEdge(e),"del_edge: is not an edge");
    CHECK(!defragmented,"del_edge: can't mutate defragmented graph!");
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
    detachFromNodes(e);
    //delete from table:
    edges(E-1)->index=e->index;
    edges(e->index)=edges(E-1);
    E--;
    edges.resizeCopy(edges.N-1);
    delete e;
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
  }

  /*! reverse the edge */
  void rev_edge(edge e){
    CHECK(checkEdge(e),"rev_edge: is not an edge");
    node f=e->from,t=e->to;
    detachFromNodes(e);
    connectToNodes(e,t,f);
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
  }

  /*! deletes all input edges of the node #n# */
  void del_in_edges(node n){
    while(n->firstIn) del_edge(n->firstIn);
  }

  /*! deletes all output edges of the node #n# */
  void del_out_edges(node n){
    while(n->firstOut) del_edge(n->firstOut);
  }

  //! delete a whole set of edges
  void del_edges(const std::list<edge>& set){
    typename std::list<edge>::const_iterator e;
    for(e=set.begin();e!=set.end();e++) del_edge_const(*e);
  }

  /*! deletes all nodes and edges */
  void clear(){
    uint i;
    if(!defragmented){
      for(i=0;i<N;i++) delete nodes(i);
      for(i=0;i<E;i++) delete edges(i);
    }else{
      if(N)  delete[] nodes(0);
      if(E) delete[] edges(0);
      //defragmented=false;
    }
    first=last=0;
    N=0;
    nodes.resize(0);
    E=0;
    edges.resize(0);
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
  }

  /*! deletes all edges */
  void clear_edges(){
    uint i;
    node n;
    for_nodes(n){
      n->firstIn=n->lastIn=n->firstOut=n->lastOut=0;
      n->Nin=n->Nout=0;
    }
    for(i=0;i<E;i++) delete edges(i);
    E=0;
    edges.resize(0);
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
  }


  //}
private:
  void connectToNodes(edge e,node from,node to){
    CHECK(!e->from && !e->to,"detach edge before connecting it to new nodes");
    e->from=from; e->to=to;
    //connecting in from's outs list:
    if(!from->Nout){
      e->prevOut=0;		  e->nextOut=0;
      from->firstOut=e;		  from->lastOut=e;
    }else{
      e->prevOut=from->lastOut;   e->nextOut=0;
      from->lastOut->nextOut=e;   from->lastOut=e;
    }
    from->Nout++;
    //connecting in to's ins list:
    if(!to->Nin){
      e->prevIn=0;		  e->nextIn=0;
      to->firstIn=e;		  to->lastIn=e;
    }else{
      e->prevIn=to->lastIn;       e->nextIn=0;
      to->lastIn->nextIn=e;       to->lastIn=e;
    }
    to->Nin++;
  }

  void detachFromNodes(edge e){
    //unlink from from's out-list:
    if(e==e->from->firstOut) e->from->firstOut=e->nextOut; else e->prevOut->nextOut=e->nextOut;
    if(e==e->from->lastOut)  e->from->lastOut =e->prevOut; else e->nextOut->prevOut=e->prevOut;
    e->from->Nout--;
    //delink from to's in-list:
    if(e==e->to->firstIn) e->to->firstIn=e->nextIn; else e->prevIn->nextIn=e->nextIn;
    if(e==e->to->lastIn)  e->to->lastIn =e->prevIn; else e->nextIn->prevIn=e->prevIn;
    e->to->Nin--;
    //reinit to zero
    e->from=e->to=0;
  }


public://{ standard looping

  /*! for(First(n);n;Next(n)) instead of for(n=first;n;n=n->next) */
  void First(node& n) const{ n=first; }
  /*! .. */
  void Next(node& n) const{ n=n?n->next:first; }
  /*! .. */
  bool CheckNext(node& n) const{ return n!=0; }
  /*! for(Last(n);n;Prev(n)) instead of for(n=last;n;n=n->prev) */
  void Last(node& n) const{ n=last; }
  /*! .. */
  void Prev(node& n) const{ n=n?n->prev:last; }
  bool CheckPrev(node& n) const{ return n!=0; }
  /*! for(FirstIn(e,n);e;NextIn(e,n)) instead of for(e=n->firstIn;e;e=e->nextIn) */
  void FirstIn(edge& e,const node n) const{ e=n->firstIn; }
  /*! .. */
  void NextIn(edge& e,const node n) const{ e=e?e->nextIn:n->firstIn; }
  /*! for(FirstOut(e,n);e;NextOut(e,n)) instead of for(e=n->firstOut;e;e=e->nextOut) */
  void FirstOut(edge& e,const node n) const{ e=n->firstOut; }
  /*! .. */
  void NextOut(edge& e,const node n) const{ e=e?e->nextOut:n->firstOut; }


public://{ graph assignment
  /*! copy operator */
  GraphT& operator=(const GraphT& G){
    node_map<node> twins;
    setGraph(G,twins);
    return *this;
  }

  /*! *this becomes a copy of g; twins stores for each node in g its
    twin in *this */
  void setGraph(const GraphT& G,node_map<node>& twins){
    node v; edge e,ne; uint i;
    twins.init(G);
    clear();
    forNodes(v,G){ twins[v]=new_node(*v); if(v->d) twins[v]->disp()=v->disp(); }
    forEdges(i,e,G){ ne=new_edge(twins[e->from],twins[e->to],e); if(e->d) ne->disp()=e->disp(); }
  }

  /*! *this becomes a copy of the subgraph containing the nodes set of
    g; twins stores for each node in set its twin in *this */
  void setSubgraph(std::list<node>& set,GraphT& G,
		   node_map<node>& twins){
    node v; edge e; uint i;
    typename std::list<node>::iterator w;
    twins.init(G);
    clear();
    forNodes(v,G) twins[v]=0;
    for(w=set.begin();w!=set.end();w++) twins[*w]=new_node(*(*w));
    forEdges(i,e,G) if(twins[e->from] && twins[e->to])
      new_edge(twins[e->from],twins[e->to],e);
  }

  /*! inserts g in *this BEFORE the node l; twins stores for each node in g its
    twin in *this */
  node insertGraph(node l,GraphT& G,node_map<node>& twins){
    node v; edge e; uint i;
    twins.init(G);
    forNodes(v,G) twins[v]=new_node(l,INSbefore,v);
    forEdges(i,e,G) new_edge(twins[e->from],twins[e->to],e);
    return twins[G.first];
  }

  /*! copy the structure of the graph g */
  template<class FGraph,class CopyStruct>
  void setForeignGraph(const FGraph& G,const CopyStruct&){ 
    node nn; edge ee; uint i;
    typename FGraph::node n; typename FGraph::edge e;
    MT::Array<node> twin; twin.resize(G.N);
    clear();
    forNodes(n,G){
      nn=new_node();
      CopyStruct::copy(*nn,*n);
      twin(n->index)=nn;
    }
    forEdges(i,e,G){
      ee=new_edge(twin(e->from->index),twin(e->to->index));
      CopyStruct::copy(*ee,*e);
    }
  }

  /*! generates a graph of #n# new nodes without connections */
  void setEmptySize(uint n){
    CHECK(n>0,"");
    clear();
    for(uint i=0;i<n;i++) new_node();
  }

  /*! generates a graph with the connectivity described by the
    connectivity matrix (containing zeros and non-zeros); #con# has
    to be 2D and either squared or n x (n+1) -dimensional */
  void setConnectivity(const MT::Array<int>& con){
    CHECK( con.nd==2 && (con.d0==con.d1 || con.d0+1==con.d1) || con.d0==con.d1+1,
	   "connectivity matrix must be squared or n x (n+1) - or (n+1) x n -dimensional");
    node v,w;
    uint N=(con.d0<=con.d1?con.d0:con.d1);
    setEmptySize(N);
    indexAsOrdered();
    for_nodes(v) for_nodes(w)
      if(con(v->index,w->index)) new_edge(w,v);
  }

  /*! sets the values of either the nodes only, the edges only, or the
    nodes and edges; in these three cases the matrix #val# has to be
    1D and n-dimensional, 2D and n x n -dimenstional, or 2D and n x
    (n+1) -dimensional, respectively. */
  template<class T>
  void setValues(const MT::Array<T>& val){
    CHECK( val.nd==1 ||
	   (val.nd==2 && (val.d0==val.d1 || val.d0+1==val.d1 || val.d0==val.d1+1)),
	   "value matrix must be squared or n x (n+1) -dimensional");
    node n; edge e; uint i;
    indexAsOrdered();
    if(val.nd==1){
      //for_nodes(n) n->set(val(n->index));
      for_nodes(n) (T&)n->data()=val(n->index);
    }
    if(val.nd==2 && val.d0==val.d1){
      for_edges(i,e) (T&)e->data()=val(e->to->index,e->from->index);
    }
    if(val.nd==2 && val.d0+1==val.d1){
      for_nodes(n) (T&)(*n)=val(n->index,val.d0);
      for_edges(i,e) (T&)e->data()=val(e->to->index,e->from->index);
    }
    if(val.nd==2 && val.d0==val.d1+1){
      for_nodes(n) (T&)(*n)=val(val.d1,n->index);
      for_edges(i,e) (T&)e->data()=val(e->to->index,e->from->index);
    }
  }


public://{ graph comparison
  //! checks for equality
  bool operator==(const GraphT& G){
    if(N!=G.N || E!=G.E) return false;
    node n,m; edge e,f;
    uint i;

    node_map<node> twin(*this);

    for(n=first,m=G.first;n && m;n=n->next,m=m->next){
      if(!(n->data()==m->data())) return false;
      if(n->Nin!=m->Nin || n->Nout!=m->Nout ) return false;
      twin[n]=m;
    }

    for_edges(i,e){
      f=G.getEdge(twin[e->from],twin[e->to]);
      if(!f) return false;
      if(!(e->data()==f->data())) return false;
    }
    return true;
  }


public://{ graph access
  /*! #con# becomes the connectivity matrix of this graph containin
    zeros and ones */
  void getConnectivity(MT::Array<int>& con) const{
    edge e; uint i;
    con.resize(N,N);
    con=0;
    node_map<uint> idx(*this);
    getOrder(idx);
    for_edges(i,e) con(idx[e->to],idx[e->from])++;
  }

  /*! #val# bexomes the n x (n+1) -dimenstional matrix containing the
    values of each node and edge */
  template<class T>
  void getValues(MT::Array<T>& val) const{
    node n; edge e; uint i;
    val.resize(N,N+1);
    val=0;
    node_map<uint> idx(*this);
    getOrder(idx);
    //for_nodes(n) n->get(val(idx[n],val.d0));
    //for_edges(i,e) e->get(val(idx[e->to],idx[e->from]));
    for_nodes(n) val(idx[n],val.d0)=(T)(*n);
    for_edges(i,e) val(idx[e->to],idx[e->from])=(T)(*e);
  }


public://{ graph manipulation
  /*! duplicates n */
  node duplicate(node n){
    CHECK(checkNode(n),"Graph::duplicate: invalid node");
    edge e; node m;
    m=new_node(n,INSafter,n);
    for_in_edges(e,n) new_edge(e->from,m,e);
    for_out_edges(e,n) new_edge(m,e->to,e);
    return m;
  }

  /*! fuses a and b */
  node fuse(node a,node b){
    CHECK(checkNode(a) && checkNode(b) && a!=b,"Graph::fuse: invalid nodes");
    edge e;
    for_in_edges(e,b) new_edge(e->from,a,e);
    for_out_edges(e,b) new_edge(a,e->to,e);
    del_node(b);
    return a;
  }


public://{ graph generation
  /*! generates a random feed-forward graph with given number of
    nodes; connections are set with probability
    Nconnections/(Nnodes*Nnodes/2); (o1, o2, and o3 are obsolete) */
  void random_graph(uint Nnodes,uint Nconnections,bool o1,bool o2,bool o3){
    uint i,j;
    double p=(double)Nconnections/(Nnodes*Nnodes/2);
    clear();
    for(i=0;i<Nnodes;i++) new_node();
    for(i=0;i<Nnodes;i++) for(j=0;j<i;j++)
      if(rnd.uni()<p){
	new_edge(nodes(j),nodes(i));
	if(o1) new_edge(nodes(i),nodes(j));
      }
  }

  /*! random FF */
  void random(uint Nnodes,uint Nconnections=0,bool symmetric=false){
    if(!Nconnections) Nconnections=rnd(Nnodes*Nnodes-Nnodes);
    random_graph(Nnodes,Nconnections,symmetric,false,false);
  }

  /*! generates a graph with #n=in+out+hid# nodes plus a plug and
    #con# connections; only feed-forward connections are installed;
    the first #in# nodes are connected to the plug with a #1#-labed
    edge; the last #out# nodes are connected to the plug with a
    #1#-labed forward; #con# gives the number of edges created and
    linked randomly; if #con# is not given, it is chosen randomly by
    con=rnd(n*n-n)
  */
  void randomFF(uint in,uint out,uint hid,uint con=0){
    uint n=in+out+hid;
    node v; edge e;
    if(!con) con=rnd(n*n-n);
    random_graph(n,con,false,true,true);
    randomUnits();
    new_node(first,INSbefore);
    n=0;
    for_nodes(v){ //delete internal inputs of input nodes
      if(n>in) break;
      if(n){
        while(v->firstIn) del_edge(v->firstIn);
	e=new_edge(first,v); //e->random(); e->label=1;
      }
      n++;
    }
    n=1;
    v=last;
    while(n<out){ v=v->prev; n++; }
    while(v!=0){
      e=new_edge(v,first);
      //e->random(); e->label=1;
      v=v->next;
    }
  }

  /*! generates a graph with #n=in+out+hid# nodes plus a plug and
    #con# connections; also recurrent connections are installed; the
    first #in# nodes are connected to the plug with a #1#-labed
    edge; the last #out# nodes are connected to the plug with a
    #1#-labed forward; #con# gives the number of edges created and
    linked randomly; if #con# is not given, it is chosen randomly by
    con=rnd(n*n-n)
  */
  void randomRecurrent(uint in,uint out,uint hid,uint con=0){
    uint n=in+out+hid;
    node v; edge e;
    if(!con) con=rnd(n*n-n);
    random_graph(n,con,false,false,true);
    randomUnits();
    new_node(first,INSbefore); //plug installation
    n=0;
    for_nodes(v){
      if(n>in) break;
      if(n){
	e=new_edge(first,v); //e->random(); e->label=1;
      }
      n++;
    }
    n=1;
    v=last;
    while(n<out){ v=v->prev; n++; }
    while(v!=0){
      e=new_edge(v,first);
      //e->random(); e->label=1;
      v=v->next;
    }
  }


  //}
private:
  void allocateDefragMem(uint nn,uint ne){
    clear();
    if(!nn) return;
    uint i;
    node NA; edge EA;
    NA=new Node [nn];
    EA=new Edge [ne];
    for(i=0;i<nn;i++){
      NA[i].index=i;
      if(i+1<nn) NA[i].next=&NA[i+1]; else NA[i].next=0;
      if(i>0)    NA[i].prev=&NA[i-1]; else NA[i].prev=0;
    }
    for(i=0;i<ne;i++){
      EA[i].index=i;
    }
    first=&NA[0];
    last =&NA[nn-1];
    N=nn;
    nodes.resize(nn);
    E=ne;
    edges.resize(ne);

    for(i=0;i<nn;i++) nodes(i)=&NA[i];
    for(i=0;i<ne;i++) edges(i)=&EA[i];

    defragmented=true;
  }
public://{ ordering and indexing
  /*! #this# becomes a defragmented copy of the graph #G#. This means
    that all the memory for nodes and edges is allocated in such a
    way that they become linearly aligned in ordinary C arrays. This
    alignment is w.r.t.\ to the INDEXING of the graph, not the
    ordering. Actually, you can do your own indexing as you want to
    specify the alignment, or use Graph::indexAsOrdered.

    In particular on Intel architectures, this makes recursion much
    faster (Up to 5 times faster for large graphs! Maybe because the
    memory can be uploaded to the cache in huge
    blocks?). E.g. recursion over all edges can now be done by
    ordinary pointer increments. Note that this is a one-way ticket:
    node/edge deletion/insertion are not possible for a defragmented
    graph and you cannot un-defragment a graph directly. You can
    though copy it into another graph, which is then un-defragmented
    (and then, if you like, copy it back into this graph) */
  void defragment(const GraphT& G){
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
    uint i,nn=G.N,ne=G.E;
    node n,F,L; edge e;
    Node *NA=0;
    Edge *EA=0;
    if(nn) NA=new Node [nn];
    if(ne) EA=new Edge [ne];
    //G.indexAllAsOrdered();
    for(i=0;i<nn;i++){
      NA[i]=*G.nodes(i);
      if(G.nodes(i)->next)     NA[i].next    =&NA[G.nodes(i)->next->index];
      if(G.nodes(i)->prev)     NA[i].prev    =&NA[G.nodes(i)->prev->index];
      if(G.nodes(i)->firstIn)  NA[i].firstIn =&EA[G.nodes(i)->firstIn->index];
      if(G.nodes(i)->lastIn)   NA[i].lastIn  =&EA[G.nodes(i)->lastIn->index];
      if(G.nodes(i)->firstOut) NA[i].firstOut=&EA[G.nodes(i)->firstOut->index];
      if(G.nodes(i)->lastOut)  NA[i].lastOut =&EA[G.nodes(i)->lastOut->index];
    }
    for(i=0;i<ne;i++){
      EA[i]=*G.edges(i);
      if(G.edges(i)->from)     EA[i].from    =&NA[G.edges(i)->from->index];
      if(G.edges(i)->to)       EA[i].to      =&NA[G.edges(i)->to->index];
      if(G.edges(i)->nextOut)  EA[i].nextOut =&EA[G.edges(i)->nextOut->index];
      if(G.edges(i)->prevOut)  EA[i].prevOut =&EA[G.edges(i)->prevOut->index];
      if(G.edges(i)->nextIn)   EA[i].nextIn  =&EA[G.edges(i)->nextIn->index];
      if(G.edges(i)->prevIn)   EA[i].prevIn  =&EA[G.edges(i)->prevIn->index];
    }

    if(G.first) F=&NA[G.first->index]; else F=0;
    if(G.last)  L=&NA[G.last->index];  else L=0;

    clear();

    first=F;
    last=L;
    N=nn;
    nodes.resize(nn);
    E=ne;
    edges.resize(ne);
    for_nodes(n){
      nodes(n->index)=n;
      for_in_edges(e,n) edges(e->index)=e;
    }
    defragmented=true;

#ifdef MT_GRAPHCHECK
    checkValid();
#endif
  }
  //! same as Graph g=*this; defragment(g);
  void defragment(){ GraphT g; g=*this; defragment(g); }

  /*! sorts the indexing (i.e. the Graph::nodes array) according to
    the ordering (as given by the node list); afterward, the
    ordering and the indexing coincide */
  void indexAsOrdered(){
    node n; uint i=0;
    for_nodes(n){ n->index=i; nodes(i)=n; i++; }
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
  }

  /*! same as Graph::indexAsOrdered, but additionally sorts the indexing
    of all edges according to their ordering (as given by the
    input-edge lists for each node) */
  void indexAllAsOrdered(){
    node n; edge e; uint i=0,j=0;
    for_nodes(n){
      n->index=i; nodes(i)=n; i++;
      for_in_edges(e,n){ e->index=j; edges(j)=e; j++; }
    }
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
  }

  /*! this function reliks the node list (i.e., changes the actual
    ordering of the nodes!) according to the order of the node
    indices (i.e., the Graph::nodes array); e.g., after topsort() or
    if you permuted some indices */
  void orderAsIndexed(){
    uint i;
    if(N){
      first=nodes(0);    first->prev=0; 
      last=nodes(N-1); last->next=0; 
      if(N>1){ first->next=nodes(1); last->prev=nodes(N-2); }
      else{ first->next=0; last->prev=0; }
      for(i=1;i<N-1;i++){
	nodes(i)->prev=nodes(i-1);
	nodes(i)->next=nodes(i+1);
      }
    }
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
  }

  /*! returns the ordering of the nodes, i.e., their position in the
      node list */
  void getOrder(node_map<uint>& order) const{
    node n; uint i=0;
    for_nodes(n){ order[n]=i; i++; }
  }

  struct SortComp{
    bool operator()(node a,node b){ return *((NodeData*)a) < *((NodeData*)b); }
  };
  /*! sorts the node indexing according to the incrementing data values */
  void sortIndexing(){
    uint i;
    node *a=(node*)nodes.p,*b=(node*)nodes.pstop;
    std::sort(a,b,SortComp());
    for(i=0;i<nodes.N;i++) nodes(i)->index=i;
  }

  //! same as sortIndexing(); orderAsIndexed();
  void sort(){ sortIndexing(); orderAsIndexed(); }

  /*! sorts the node indexing according to the graph's topology. False
      is returned and nothing happens if the graph has cycles. */
  bool topsortIndexing(){ 
    std::list<node> noIns;
    node_map<int> ins(*this);

    uint count=0;
    node v;
    edge e;

    for_nodes(v){
      ins[v]=v->Nin;
      if(ins[v]==0) noIns.push_front(v);
    }

    while(!noIns.empty()){
      v=noIns.front();
      noIns.pop_front();
      v->index=count++;
      for_out_edges(e,v)
	if(--ins[e->to]==0) noIns.push_front(e->to);
    }
  
    if(count!=N){
      for(uint i=0;i<N;i++) nodes(i)->index=i;
      return false;
    }

    for_nodes(v) nodes(v->index)=v;
    return true;
#ifdef MT_GRAPHCHECK
    checkValid();
#endif
  }

  //! same as topsortIndexing(); orderAsIndexed();
  bool topsort(){ bool ts=topsortIndexing(); orderAsIndexed(); return ts; }

  /*! calculates the layer for each nodes and stores the layer number
    in the given node_map */
  uint getLayers(node_map<uint>& layer){
    node n; edge e;
    uint l=1;
    layer.init(*this);
    for_nodes(n) layer[n]=0;
    for_nodes(n){
      for_in_edges(e,n){
	if(layer[e->from]==l){
	  l++;
	  break;
	}
      }
      layer[n]=l;
    }
    for_nodes(n) layer[n]--;
    return l;
  }

  /*! labels the clusters of the graph -- PRECONDITION: node and edge
    classes have a field `label' */
  int labelClusters(){
    node n;
    for_nodes(n) n->label=0;
    int l=0;
    for_nodes(n) if(!n->label){
      l++;
      labelSubCluster(n,l);
    }
    return l;
  }


  //}
private:
  void labelSubCluster(node n,int label){
    n->label=label;
    edge e;
    for_out_edges(e,n){
      e->label=label;
      if(!e->to->label) labelSubCluster(e->to,label);
    }
    for_in_edges(e,n){
      e->label=label;
      if(!e->from->label) labelSubCluster(e->from,label);
    }
  }

  
public://{ misc / obsoletes
  void addPlug(uint in,uint out){
    CHECK(in+out<=N,"in+out greater than number of units");
    uint i,nn=N;
    new_node(first,INSbefore);
    for(i=0;i<in;i++) new_edge(first,nodes(i));
    for(i=nn-out;i<nn;i++) new_edge(nodes(i),first);
  }

  /*! set connection matrix and parameters (weights) in ReClaM
    conventions */
  void setReClaM(uint in,uint out,const MT::Array<int>& mat,const MT::Array<double>& par){
    setConnectivity(mat);
    setValues(mat);
    setValues(par);
    addPlug(in,out);
  }

  /*! ... */
  void setCoreVector(MT::Array<int>& vec);

  /*! if #N!=mat.d0+1#, then #setEmptySize(mat.d0)# is called! */
  void setCoreMatrix(const MT::Array<int>& mat){
    node v,w; edge e; int l;
    setEmptySize(mat.d0+1);
    indexAsOrdered();
    for_nodes(v) if(v->index!=0){
      for_nodes(w) if(w!=v && w->index!=0){
	l=mat(v->index-1,w->index-1);
	if(l){ e=new_edge(v,w); e->label=l; }
      }
      v->label=mat(v->index-1,v->index-1);
    }
  }


  /*! generates a completely forward-connected layer structure with
    #size(i)# nodes in the #i#th layer; first and last layers are
    input and output, respectively; if edit is true, the layer nodes
    and edges can be edited before they are multiplied.
  */
  void setLayer(MT::Array<uint>& size,bool edit=false){
    CHECK(size.nd==1 && size.d0>1,"need at least two layers to generate graph");
    uint i,j;
    node n;
    clear();
    n=new_node(); //plug
    for(i=0;i<size.N;i++){
      n=new_node();
      new_edge(n->prev,n);
      if(i==size.d0-1){ //last layer
	new_edge(n,first); //out to plug
      }
    }
    //if(edit) watch();
    n=first;
    for(i=0;i<size.N;i++){
      n=n->next;
      for(j=1;j<size(i);j++){ duplicate(n); n=n->next; }
    }
  }
  void setLayer(uint in,uint hid,uint out,int inlabels,int outlabels){
    MT::Array<uint> layers;
    layers.resize(3);
    layers(0)=in; layers(1)=hid; layers(2)=out;
    setLayer(layers);
    randomUnits();
    setInLabels(inlabels);
    setOutLabels(outlabels);
  }


  /*! the matrix excluding the plug */
  void getCoreMatrix(MT::Array<int>& mat){
    node n; edge e;
    node_map<uint> idx(*this);
    mat.resize(N-1,N-1);
    mat=0;
    indexAsOrdered();
    for_nodes(n) if(n->index!=0){
      for_out_edges(e,n) if(e->to->index!=0)
	mat(n->index-1,e->to->index-1)|=e->label;
      mat(n->index-1,n->index-1)=n->label;
    }
  }

  /*! the vector excluding the plug */
  void getCoreVector(MT::Array<int>& data){
    node n;
    node_map<uint> idx(*this);
    data.resize(N-1);
    indexAsOrdered();
    for_nodes(n) if(n->index!=0){ data(n->index-1)=n->label; }
  }
  void randomUnits(){
    node v; edge e; uint i;
    for_nodes(v){ v->random(); }
    for_edges(i,e){ e->random(); }
  }
  /*! sets the labels of all output nodes */
  void setOutLabels(int l){
    edge e;
    for_in_edges(e,first) e->from->label=l;
  }
  /*! sets the labels of all input nodes */
  void setInLabels(int l){
    edge e;
    for_out_edges(e,first) e->to->label=l;
  }

  /*! only for debugging : checks internal lists and tables of nodes
    and edges for consistency */
  bool checkValid() const{
    node n; edge e; uint i;

    CHECK(N==nodes.N,"");
    CHECK(E==edges.N,"");

    //check node table:
    for(i=0;i<N;i++) CHECK(nodes(i)->index==i,"");

    //check edge table:
    for(i=0;i<E;i++) CHECK(edges(i)->index==i,"");

    //check node list:
    uint num=0,nume=0;
    for_nodes(n){
      if(n->prev==0){ CHECK(first==n,""); }else CHECK(n->prev->next==n,"");
      if(n->next==0){ CHECK(last ==n,""); }else CHECK(n->next->prev==n,"");
      num++;
    }
    CHECK(num==N,"");

    //check out/in lists:
    for_nodes(n){
      num=0;
      for_out_edges(e,n){
	CHECK(e->from==n,"");
	if(e->prevOut==0){ CHECK(n->firstOut==e,""); }else CHECK(e->prevOut->nextOut==e,"");
	if(e->nextOut==0){ CHECK(n->lastOut ==e,""); }else CHECK(e->nextOut->prevOut==e,"");
	num++;
      }
      CHECK(num==n->Nout,"");
      nume+=num;
      num=0;
      for_in_edges(e,n){
	CHECK(e->to==n,"");
	if(e->prevIn==0){ CHECK(n->firstIn==e,""); }else CHECK(e->prevIn->nextIn==e,"");
	if(e->nextIn==0){ CHECK(n->lastIn ==e,""); }else CHECK(e->nextIn->prevIn==e,"");
	num++;
      }
      CHECK(num==n->Nin,"");
      nume+=num;
    }
    CHECK(nume==2*E,"");

    return true;
  }

  //}
private:
  void edgepos(const edge& e,float& x0,float& y0,
	       float& x1,float& y1,
	       float& dx,float& dy,float o=0.) const{
    float l;
    dx=e->to->disp().x - e->from->disp().x; dy=e->to->disp().y - e->from->disp().y;
    l=sqrt(dx*dx+dy*dy);
    if(l<.0001){ dx=0.; dy=-e->to->disp().size; }
    else{ dx*=e->to->disp().size/l; dy*=e->to->disp().size/l; }
    x0=e->from->disp().x+dx-o*dy; y0=e->from->disp().y+dy+o*dx;
    x1=e->to->disp().x  -dx-o*dy; y1=e->to->disp().y  -dy+o*dx;
  }

  void spring(){
    edge e; uint i;
    node n,v;
    NodeDisp *f,*fb;
    float dx,dy,d;
    uint c;
    for_nodes(n){
      f=&(n->disp());
      f->fx=f->fy=0.;
      if(!f->x && !f->y){
	f->x=rnd.uni(-.8,.8);
	f->y=rnd.uni(-.8,.8);
      }else{
	f->x+=rnd.gauss(.001);
	f->y+=rnd.gauss(.001);
      }
    }

    for(c=0;c<1;c++){
      //attraction:
      for_edges(i,e){
	f=&(e->to->disp());
	fb=&(e->from->disp());
	dx=fb->x-f->x;
	dy=fb->y-f->y;
	d=::sqrt(dx*dx+dy*dy);
	if(d<.1) d=.1;
	f->fx+=(d-.3)*dx/d;
	f->fy+=(d-.3)*dy/d;
	fb->fx-=(d-.3)*dx/d;
	fb->fy-=(d-.3)*dy/d;
      }

      //repulsion:
      for_nodes(n){
	f=&(n->disp());
	//from each other
	for_nodes(v) if(n!=v){
	  fb=&(v->disp());
	  dx=fb->x-f->x;
	  dy=fb->y-f->y;
	  d=::sqrt(dx*dx+dy*dy);
	  if(d<.1) d=.1;
	  f->fx+=-.01*dx/(d*d*d);
	  f->fy+=-.01*dy/(d*d*d);
	  fb->fx-=-.01*dx/(d*d*d);
	  fb->fy-=-.01*dy/(d*d*d);
	}
      
	//from border
	f->fx-=.1*f->x; //*f->x*f->x;
	f->fy-=.1*f->y; //*f->y*f->y;
      }
      //moving:
      for_nodes(n){
	f=&(n->disp());
	f->x+=.03*f->fx;
	f->y+=.03*f->fy;
      }
    }
  }
  void circle(){
    node v;
    uint i=0;
    double sec=2.0*MT_PI/N,r=0.8;
    for_nodes(v){
      v->disp().x=r*::cos(i*sec);
      v->disp().y=r*::sin(i*sec);
      i++;
    }
  }
  void layer(){
    node v;
    uint n,N,L=0;
    double x,y;
    node_map<uint> lay;

    L=getLayers(lay);
    for(uint l=0;l<L;l++){
      y=(double)(l+1)/(L+1); y=2.*y-1.;
      N=0;
      for_nodes(v) if(lay[v]==l && !v->disp().hide) N++;
      n=0;
      for_nodes(v) if(lay[v]==l && !v->disp().hide){
	x=(double)(n+1)/(N+1); x=2.*x-1.;
	v->disp().x=x;
	v->disp().y=y;
	n++;
      }
    }
  }
  void layer2D(){
    node v;
    uint n,N,L,N2=0;
    double x,y,dy,dx;
    node_map<uint> lay;

    L=getLayers(lay);
    for(uint l=0;l<L;l++){
      y=(double)(l+1)/(L+1); y=2.*y-1.;
      N=0;
      for_nodes(v) if(lay[v]==l) N++;
      N2=(uint)floor((double)sqrt((double)N));
      n=0;
      for_nodes(v) if(lay[v]==l){
	if(l==0){
	  dy=floor((double)n/N2);
	  dx=(n-dy*N2);           
	  dy=.8*(dy+1)/(N2+1)/(L+2);
	  dx=(double)(dx+1)/(N2+1)+.5*dy;
	  /*
	    dy*=0.8/N2;        dx+=1+dy;
	    dy*=(y2-y1)/(L+1); dx*=(x2-x1)/(N2+1);
	    xpos[v]=x1+dx;
	    ypos[v]=y+dy;
	  */
	  v->disp().x=2.*dx-1.;
	  v->disp().y=y+dy;
	}else{
	  x=(double)(n+1)/(N+1); x=2.*x-1.;
	  v->disp().x=x;
	  v->disp().y=y;
	}
	n++;
      }
    }
  }
#if 0
  void layer2D(,uint N2){
//static Parameter<uint> GraphShare::graphDisplayX("graphDisplayX");
    node_map<double> xpos(*this);
    node_map<double> ypos(*this);
    node v; point p;
    uint n,N,L;
    //,N2=0;
    double x,y,dy,dx;

    L=getLayer();
    for(uint l=0;l<L;l++){
      y=y1+(l+1)*(y2-y1)/(L+1);
      N=0;
      for_nodes(v) if(indices[v]==l) N++;
      //N2=(uint)floor((double)sqrt((double)N));
      n=0;
      for_nodes(v) if(indices[v]==l){
	if(l==1){
	  dy=floor((double)n/N2);    dx=(n-dy*N2);
	  /*
	    dy*=0.8/N2;        dx+=1+dy;
	    dy*=(y2-y1)/(L+1); dx*=(x2-x1)/(N2+1);
	    xpos[v]=x1+dx;
	    ypos[v]=y+dy;
	  */
	  xpos[v]=x1+(dx+1)*(x2-x1)/(N2+1);
	  ypos[v]=y1+(dy+1)*(y2-y1)/((N/N2)+1);
	}else{
	  x=x1+(n+1)*(x2-x1)/(N+1);
	  //xpos[v]=x;
	  //ypos[v]=y;
	}
	n++;
      }
    }
    spring(x1,x2,y1,y2);
  }
#endif

public://{ I/O: methods for OpenGL, postscript, and FIG
  //! layout of the graph
  void layoutGraph(){
    switch(graphLayout){
    case 0: break;
    case 1: circle(); break;
    case 2: layer(); break;
    case 3: spring(); break;
    case 4: layer2D(); break;
      //case 4: layer2D(graphDisplayX); break;
    }
  }

#ifdef MT_QT
  static void staticDraw(void* classP){ ((Graph*)classP)->glDraw(); }
  //! the OpenGL draw routine
  void glDraw(){
    node n; edge e;
    float x0,y0,x1,y1,dx,dy;
    NodeDisp *f;

    layoutGraph();

    /*if(hideGraphPlug){
      edge e;
      for_in_edges(e,first) e->disp().hide=true;
      for_out_edges(e,first) e->disp().hide=true;
    }*/

    glPushName(13);

    for_nodes(n){
      f=&(n->disp());
      for_in_edges(e,n) if(!e->disp().hide){
	glLoadName(N+e->index);
	edgepos(e,x0,y0,x1,y1,dx,dy,.3);
	glBegin(GL_LINES);
	glColor3f(e->disp().col.r,e->disp().col.g,e->disp().col.b);
	glVertex3f(x0,y0,e->from->disp().z);
	glVertex3f(x1,y1,f->z);
	glEnd();
	glBegin(GL_TRIANGLES);
	glColor3f(e->disp().col.r,e->disp().col.g,e->disp().col.b);
	glVertex3f(x1         ,y1         ,f->z);
	glVertex3f(x1-dx-.5*dy,y1-dy+.5*dx,f->z);
	glVertex3f(x1-dx+.5*dy,y1-dy-.5*dx,f->z);
	glEnd();
	/*if(drawText){
	  String s;
	  e->write(s); // <<*e;
	  glDrawText(s,.7*x0+.3*x1,.7*y0+.3*y1,.7*e->from->disp().z+.3*f->z);
	}*/
	Optionals::glDrawEdge(e,x0,y0,e->from->disp().z,x1,y1,f->z);
      }
      if(!f->hide){
	glLoadName(n->index);
	glDrawRect(-f->size+f->x,-f->size+f->y,.01+f->z,
		   f->size+f->x,-f->size+f->y,.01+f->z,
		   f->size+f->x, f->size+f->y,.01+f->z,
		   -f->size+f->x, f->size+f->y,.01+f->z,
		   f->col.r,f->col.g,f->col.b);
	/*if(drawText){
	  String s;
	  n->write(s); // <<*n;
	  glDrawText(s,f->x,f->y,.01+f->z);
	}*/
	Optionals::glDrawNode(n,f->x,f->y,f->z);
      }
    }

    glPopName();
  }

  //! writes info on selected nodes/edges on std::cout
  void writeSelected(GLuint *selected){
    uint i=selected[3];
    CHECK(i<N+E,"dammit!");
    if(i<N){
      std::cout <<"node watch [" <<i <<"]: " <<*nodes(i) <<std::endl;
    }else{
      i-=N;
      std::cout <<"edge watch [" <<i<<":"<<edges(i)->from->index<<"->"<<edges(i)->to->index<<"]: " <<*edges(i) <<std::endl;
    }
  }
#endif

#if 0
  /*! display via OpenGL */
  void display(void *parent=0,const char* name=0,uint width=400,uint height=400,int posx=0,int posy=0){
#ifdef MT_QT
    if(!opengl){
      MT::qt.init();
      opengl=new OpenGL((QWidget*)parent,name,width,height,posx,posy);
      opengl->add(this,staticDraw);
      opengl->camera.setPosition(0.,0.,10.);
      opengl->camera.focusOrigin();
      opengl->camera.setZRange(1.,100.);
      opengl->camera.setHeightAngle(12.);
    }

    static char tit[100];
    sprintf(tit,"<Graph nodes=%u edges=%u> %s",N,E,name);
    opengl->caption=tit;
    opengl->show();
    opengl->update();
#endif
  }

  /*! closes and deallocates display */
  void undisplay(){
#ifdef MT_QT
    if(opengl) delete opengl;
    opengl=0;
#endif
  }

  /*! displays and waits until ENTER stroke in the display */
  bool watch(const char *name=""){
#ifdef MT_QT
    display(0,name);
    opengl->watch();
    while(opengl->selected){
      uint i=opengl->selected[3];
      CHECK(i<N+E,"dammit!");
      if(i<N){
	std::cout <<"node watch [" <<i <<"]: " <<*nodes(i) <<std::endl;
      }else{
	i-=N;
	edge e=edges(i);
	std::cout <<"edge watch [" <<i<<":"<<e->from->index<<"->"<<e->to->index<<"]: " <<*e <<std::endl;
      }
      opengl->selected=0;
      opengl->watch();
    }
    return false;
#endif
    return true;
  }
#endif

  /*! saves a fig-file (for xfig) of the graph -- use this to convert
    to postscript etc. */
  void saveFIG(const char* name){
    layoutGraph();
    std::ofstream os(name);
    CHECK(os.good() && !os.bad(),"could not open file '" <<name <<"' for output");
    node n; edge e; uint i;
    float x,y,x2,y2,dx,dy,r;
    os <<"#FIG 3.2\nLandscape\nCenter\nInches\nLetter\n100.00\nSingle\n-2\n1200 2"
       <<std::endl;
    os <<"0 32 #000000\n0 33 #101010\n0 34 #202020\n0 35 #303030\n0 36 #404040\n0 37 #505050\n0 38 #606060\n0 39 #707070\n0 40 #808080\n0 41 #909090\n0 42 #a0a0a0\n0 43 #b0b0b0\n0 44 #c0c0c0\n0 45 #d0d0d0\n0 46 #e0e0e0\n0 47 #f0f0f0\n0 48 #ffffff" <<std::endl;
    for_nodes(n) if(!n->disp().hide){
      x=floor(5000.*n->disp().x); y=-floor(5000.*n->disp().y); r=ceil(5000.*n->disp().size);
      if(!n->disp().shape){ //circle
	os <<"1 3 0 0 32 40 50 -1 "
	   <<(int)(40.*n->disp().col.getGray())
	   <<" 0.000 1 0.0000 "
	   <<x <<" " <<y <<" " <<r <<" " <<r <<" "
	   <<x <<" " <<y <<" " <<x+r <<" " <<y+r <<std::endl;
      }else{ //box
	os <<"2 2 0 0 32 40 50 -1 " //pen color=32, fill color=40, layer=50
	   <<(int)(40.*n->disp().col.getGray()) //fill density
	   <<" 0.000 0 0 7 0 0 5\n\t"
	   <<x-r <<" " <<y-r <<" " <<x+r <<" " <<y-r <<" "
	   <<x+r <<" " <<y+r <<" " <<x-r <<" " <<y+r <<" "
	   <<x-r <<" " <<y-r <<std::endl;
      }
    }
    for_edges(i,e) if(!e->disp().hide){
      edgepos(e,x,y,x2,y2,dx,dy);
      x=floor(5000.*x);y=-floor(5000.*y);x2=floor(5000.*x2);y2=-floor(5000.*y2);
      os <<"2 1 0 1 "
	 <<32+(int)(16.*e->disp().col.getGray()) //pen color
	 <<" -1 " //fill color
	 <<60+(int)(16.*e->disp().col.getGray()) //layer
	 <<" -1 -1 0.000 0 0 -1 1 0 2\n\t"
	 <<"1 1 1.00 100.00 200.00\n\t"
	 <<x <<" " <<y <<" " <<x2 <<" " <<y2 <<std::endl;
    }
    os.close();
  }

  /*! loads graph from a ReClaM file -- only with MT_SHARK flag! */
  void loadReClaM(const char* file){
#if defined MT_SHARK && defined MSEFFNET_H
    MSEFFNet net(file);
    setReClaM(net.inputDimension,net.outputDimension,
	      net.getConnections(),net.getWeights());
#else
    HALT("can't loadReClam without <ReClaM/MSEFFNet.h> loaded before <MT/graph.h>");
#endif
  }

  /*! temperature coloring of nodes and edges, scaled to min/max-values */
  void colorTempMinMax(bool printminmax=false){
    node n;
    uint i; edge e;
    if(!N) return;
    double min,max;
    min=max=first->disp().c;
    for_nodes(n){ if(n->disp().c>max) max=n->disp().c; if(n->disp().c<min) min=n->disp().c; }
    for_nodes(n){ n->disp().col.setTemp((n->disp().c-min)/(max-min)); }
    if(printminmax) std::cout <<min <<" " <<max <<" " <<std::endl;
    if(E) min=max=edges(0)->disp().c;
    for_edges(i,e){ if(e->disp().c>max) max=e->disp().c; if(e->disp().c<min) min=e->disp().c; }
    for_edges(i,e){ e->disp().col.setTemp((e->disp().c-min)/(max-min)); }
  }

  /*! temperature coloring of nodes and edges, scaled to min/max-values */
  void colorTemp(){
    uint i; node n; edge e;
    if(!N) return;
    for_nodes(n){ n->disp().col.setTemp(n->disp().c); }
    for_edges(i,e){ e->disp().col.setTemp(e->disp().c); }
  }

  /*! prototype for #operator<<# */
  void write(std::ostream& os,bool topOnly=false,bool streamFeatures=false) const{
    node n; edge e;
    node_map<uint> idx(*this);
    os <<"<Graph nodes=" <<N <<" edges=" <<E <<">\n";
    if(topOnly){
      for_nodes(n) os <<n->index <<' '; os <<'\n';
      for_nodes(n) for_in_edges(e,n) os <<e->from->index <<"->"<<e->to->index <<'\n';
    }else if(streamFeatures){
      for_nodes(n) os <<n->index <<" [" <<(*n) <<" [" <<n->disp() <<"]];\n";
      for_nodes(n) for_in_edges(e,n) os <<e->from->index <<"->"<<e->to->index <<" [" <<(*e) <<"];\n";
    }else{
      for_nodes(n) os <<n->index <<" [" <<(*n) <<"];\n";
      for_nodes(n) for_in_edges(e,n) os <<e->from->index <<"->"<<e->to->index <<" [" <<(*e) <<"];\n";
    }
    os <<"</Graph>" <<std::endl;
  }

  /*! prototype for #operator>># */
  void read(std::istream& is,bool topOnly=false,bool streamFeatures=false){
#ifndef OLD_GRAPH_READ
    uint i,j,k,nn,ne;
    edge e;
    is >>"<Graph" >>"nodes=" >>nn >>"edges=" >>ne >>">";
    CHECK(is.good(),"could not read graph tag");
    clear();
    if(!defragmented){
      setEmptySize(nn);
      for(i=0;i<nn;i++){
	if(!streamFeatures){ is >>j; is >>"[" >>*nodes(j) >>"];"; }
	else{ is >>j; is >>"[" >>*nodes(j) >>"[" >>nodes(j)->disp() >>"]];"; }
	CHECK(is.good(),"could not read the "<<i<<"th node of graph");
      }
      for(i=0;i<ne;i++){
	is >>j >>"->" >>k >>"[";
	e=new_edge(nodes(j),nodes(k));
	is >>(*e) >>"];";
	CHECK(is.good(),"could not read the "<<i<<"th edge of graph");
      }
    }else{
      allocateDefragMem(nn,ne);
      for(i=0;i<nn;i++){
	if(!streamFeatures){ is >>j; is >>"[" >>*nodes(j) >>"];"; }
	else{ is >>j; is >>"[" >>*nodes(j) >>"[" >>nodes(j)->disp() >>"]];"; }
	CHECK(is.good(),"could not read the "<<i<<"th node of graph");
      }
      for(i=0;i<ne;i++){
	is >>j >>"->" >>k >>"[";
	connectToNodes(edges(i),nodes(j),nodes(k));
	is >>(*edges(i)) >>"];";
	CHECK(is.good(),"could not read the "<<i<<"th edge of graph");
      }
    }
    is >>"</Graph>";
    CHECK(is.good(),"could not read graph stop tag");
#else
    MT::Array<int> A; MT::Array<double> B;
    is >>A;
    if(is.fail()) return;
    setConnectivity(transpose(A));
    setValues(transpose(A));
    is >>(const char*)"<weights>";
    if(!is.fail()){
      is >>B;
      if(!samedim(A,B))
	HALT("weight matrix has to have same dimension as connection m");
      setValues(B);
    }else{
      is.clear();
    }
#endif
  }
  //}
};
template<class N,class E,class O>
inline std::istream& operator>>(std::istream& is,Graph<N,E,O>& x){ x.read(is);return is; }
template<class N,class E,class O>
inline std::ostream& operator<<(std::ostream& os,const Graph<N,E,O>& x){ x.write(os); return os; }


//===========================================================================
//
// primitive type graph
//

/*! This class allows to use the Graph class with primitive types
  (i.e., non-structs/classes like int, double, etc). You have to refer
  to the node and edge data via n->x or e->x */
template<class NodeData,class EdgeData>
class Graph_:public Graph<basetype<NodeData>,basetype<EdgeData> >{
public:
  typedef Graph<basetype<NodeData>,basetype<EdgeData> > GraphT;
  typedef typename GraphT::node node;
  typedef typename GraphT::edge edge;
  node new_node(node where=0,int opt=INSafter,const NodeData* x=0){
    if(!x) return GraphT::new_node(where,opt,0);
    return GraphT::new_node(where,opt,&basetype<NodeData>(*x));
  }
  node new_node(const NodeData& x){ return GraphT::new_node(basetype<NodeData>(x)); }
  /*
    edge new_edge(node from,node to,const EdgeData* x=0){
    if(!x) return new_edge(from,to,0);
    return new_edge(from,to,&basetype<EdgeData>(*x));
    }*/
};

//#define graph Graph_


//===========================================================================
//
// algorithms on graphs
//

template<class Node,class Edge,class opts,class CompareOp>
void maximumSpanningTree(Graph<Node,Edge,opts>& g,const CompareOp& cmp){
  uint i;
  typedef Graph<Node,Edge,opts> graphT;
  typename graphT::node n;
  typename graphT::edge e,m;
  typename graphT::template node_map<bool> treeNode; treeNode.init(g);
  typename graphT::template edge_map<bool> treeEdge; treeEdge.init(g);

  if(!g.N) return;
  forNodes(n,g) treeNode[n]=false;
  forEdges(i,e,g) treeEdge[e]=false;

  uintA treeNodes;
  
  n=g.rndNode();
  treeNodes.append(n->index); treeNode[n]=true;
  while(treeNodes.N<g.N){
    m=0;
    for(i=0;i<treeNodes.N;i++){
      n=g.nodes(treeNodes(i));
      for_out_edges(e,n) if(!treeNode[e->to]   && (!m || cmp(e,m))) m=e;
      for_in_edges(e,n)  if(!treeNode[e->from] && (!m || cmp(e,m))) m=e;
    }
    CHECK(m,"graph is not connected!");
    treeEdge[m]=true;
    if(treeNode[m->to]) g.rev_edge(m);
    treeNode[m->to]=true;
    treeNodes.append(m->to->index);
  }
  forEdges_rev(i,e,g) if(!treeEdge[e]) g.del_edge(e);
}
#endif
