#include "array.h"

void gnuplot(const MT::Array<double>& X){
  if(X.nd==2 && X.d1!=2){ //assume array -> splot
    MT::IOraw=true;
    MT::save(X,"pltX");
    gnuplot("splot 'pltX' matrix");
  }
  if(X.nd==2 && X.d1==2){ //assume curve -> plot
    MT::IOraw=true;
    MT::save(X,"pltX");
    gnuplot("plot 'pltX' us 1:2");
  }
  if(X.nd==1){ //assume curve -> plot
    MT::IOraw=true;
    MT::Array<double> Y;
    Y.referTo(X);
    Y.resize(Y.N,1);
    MT::save(Y,"pltX");
    gnuplot("plot 'pltX' us 1");
  }
}

void write(const MT::Array<double>& X,MT::Array<double>& Y,const char* name){
  std::ofstream os;
  MT::open(os,name);
  MT::IOraw=true;
  if(X.nd==1){
    for(uint i=0;i<X.N;i++) os <<X(i) <<' ' <<Y(i) <<std::endl;
  }else{
    for(uint i=0;i<X.N;i++) os <<X[i] <<' ' <<Y[i] <<std::endl;
  }
}

#ifdef MT_EXPRESSIONS
void assign(MT::Array<double>& x){
  CHECK(x.ex,"self-assignment only if it is an expression");
  MT::Ex *e=x.ex;
  x.init();
  x.ex=e;
  assign(x,x);
  delete x.ex;
  x.ex=0;
}

void assign(MT::Array<double>& x,const MT::Array<double>& a){
  if(!a.ex){ x=a; return; }
  MT::Ex &e=*a.ex;
  if(e.op==MT::UNI){
    MT::Array<double> *A=(MT::Array<double>*)e.A;
    if(A->ex) assign(*A);
    if(!e.trans && e.mul==1 && e.add==0 ){ x=*A; return; }
    if(!e.trans && e.mul==1 ){ scalarPlus(x,*A,*((double*)&e.add)); return; }
    if(!e.trans && e.add==0 ){ scalarMultiplication(x,*A,*((double*)&e.mul)); return; }
    if(e.mul==1 && e.add==0 ){ transpose(x,*A); return; }
    HALT("");
  }else{
    MT::Array<double> *A=(MT::Array<double>*)e.A,*B=(MT::Array<double>*)e.B;
    if(A->ex) assign(*A);
    if(B->ex) assign(*B);
    //bool at,bt;
    //double ac,bc,ap,bp;
    switch(e.op){
    case MT::PROD:
      if(!A->ex && !B->ex){ innerProduct(x,*A,*B); return; }
      HALT("prod");
      break;
    case MT::MUL:
      if(!A->ex && !B->ex){ mult(x,*A,*B); return; }
      HALT("mult");
      break;
    case MT::Div:
      if(!A->ex && !B->ex){ div(x,*A,*B); return; }
      HALT("mult");
      break;
    case MT::OUT:
      if(!A->ex && !B->ex){ outerProduct(x,*A,*B); return; }
      HALT("out");
      break;
    case MT::PLUS:
      if(!A->ex && !B->ex){ plus(x,*A,*B); return; }
      //if(A->ex){ ap=A->ex->add; ac=A->ex->mul; at=A->ex->trans; A=(MT::Array<double>*)A->ex->A; }else{ ap=0; ac=1; at=false; }
      //if(B->ex){ bp=B->ex->add; bc=B->ex->mul; bt=B->ex->trans; B=(MT::Array<double>*)B->ex->A; }else{ bp=0; bc=1; bt=false; }
      //if(!at && !bt && !ap && !bp){ plus(x,ac,*A,bc,*B); return; }
      //if(!at && !bt && !B){ scalarPlus(x,*A,bc); return; }
      HALT("plus");
      break;
    case MT::MINUS:
      if(!A->ex && !B->ex){ minus(x,*A,*B); return; }
      //if(A->ex){ ap=A->ex->add; ac=A->ex->mul; at=A->ex->trans; A=(MT::Array<double>*)A->ex->A; }else{ ap=0; ac=1; at=false; }
      //if(B->ex){ bp=B->ex->add; bc=B->ex->mul; bt=B->ex->trans; B=(MT::Array<double>*)B->ex->A; }else{ bp=0; bc=1; bt=false; }
      //if(!at && !bt && !ap && !bp){ plus(x,ac,*A,-bc,*B); return; }
      //if(!at && !bt && !B){ scalarPlus(x,*A,bc); return; }
      HALT("minus");
      break;
    case MT::UNI:
      HALT("shouldn't be here!");
      break;
    }
    HALT("yet undefined expression");
  }
}
#endif
