#include"testFunctions.h"

static uint HXORval=0;
intA TestFunctions::kTrapMix; 

bool HXORsub(byte* s,uint n){
  uint i,l=MT::Log2(n);
  
  bool valL,valR;
  if(!n) return true;
  if(n==1){ HXORval+=1; return true; }
  if(n==(uint)(1<<l)){ //length is a power of 2
    valL=HXORsub(s,n/2);
    valR=HXORsub(s+n/2,n/2);
    if(!valL || !valR) return false;
    for(i=0;i<n/2;i++) if((s[i]&1)==(s[i+n/2]&1)) return false;
    HXORval+=n;
  }else{
    i=1<<l;
    HXORsub(s,i); HXORsub(s+i,n-i);
    return false;
  }
  return true;
}

//! the hierarchical XOR function for a string of 0/1s
double TestFunctions::HXOR(byte* s,uint n,bool cleanFromNon01){
  HXORval=0;
  if(cleanFromNon01){ //clean from all non-0s/1s
    uint i,j;
    for(i=0,j=0;i<n;i++){ /*s[i]-=lowestSymbol;*/ s[j]=s[i]; if(s[i]==0 || s[i]==1) j++; }
    n=j;
  }
  HXORsub(s,n);
  return HXORval;
}

//! the hierarchical XOR function for a string of 0/1s
double TestFunctions::HXOR(const MT::Array<int>& x,uint cutoff,bool cleanFromNon01){
  MT::Array<byte> s;
  if(!cutoff) cutoff=x.N;
  s.resize(x.N<cutoff?x.N:cutoff);
  for(uint i=0;i<s.N;i++) s.elem(i)=(byte)x.elem(i);
  double f=HXOR(s.p,s.N,cleanFromNon01);
  if(cutoff) f/=(cutoff*(1+MT::Log2(cutoff)));
  return f;
}

//! counts the number of 1s (or targetSymbols) in a string
double TestFunctions::CountingOnes(const MT::Array<byte>& x,byte targetSymbol){
  uint i,c=0;
  for(i=0;i<x.N;i++) if(x(i)==targetSymbol) c++;
  return c;
}

//! counts the number of 1s (or targetSymbols) in a string
double TestFunctions::CountingOnes(const MT::Array<int>& x,uint cutoff,int targetSymbol){
  uint i,c=0;
  for(i=0;i<x.N && i<cutoff;i++) if(x(i)==targetSymbol) c++;
  return ((double)c)/cutoff;
}

double subkTrap(int* x,uint n,uint k){
  CHECK(n<=k,"kTrap subfunction needs input length <= k");
  if(!n) return 0;
  uint i;
  uint c[2];
  c[0]=c[1]=0;
  for(i=0;i<n;i++) c[x[i]]++;
  if(c[1]==k) return 1.;
  if(!c[0]) return 0.;
  return double(c[0]-1)/k;
}
double TestFunctions::kTrap(const intA& x,uint k,uint L){
  double f=0.;
  CHECK(L>=k && !(L%k),"total length in kTraps needs to be multiple of k");
  uint i,N=x.N; if(N>L) N=L;
  
  //transform the optimum to a random sequence instead of all 1s -- via XOR with random sequence
  static intA y;
  if(kTrapMix.N!=L){ kTrapMix.resize(L); rndNum(kTrapMix,0,1); }
  y=x;
  for(i=0;i<N;i++) y(i)^=kTrapMix(i);

  for(i=0;i+k-1<N;i+=k) f+=subkTrap(&y(i),k,k); //evaluate the full blocks
  if(i<N) f+=subkTrap(&y(i),N-i,k);             //evaluate the last partial block

  return f/(L/k);  //rescaling: maximum is 1.
}

void TestFunctions::LocalLinearSwitch::instantiate(uint inDim,uint outDim,uint nModels){
  CHECK(inDim && outDim && nModels,"dimensions and number of models must be non-zero");
  M.resize(nModels);
  uint i,j;
  for(i=0;i<nModels;i++){
    M(i).cen.resize(inDim);	       rndUni(M(i).cen,-1,1);
    M(i).off.resize(outDim);       rndUni(M(i).off,-1,1);
    M(i).mat.resize(outDim,inDim); rndUni(M(i).mat,-1,1);
    M(i).sig=rnd.uni(.01,.1);
  }
  if(nModels>2){ //make two of the equal!
    i=rnd(nModels); j=rnd(nModels-1); if(j>=i) j++;
    M(i).mat=M(j).mat;
    M(i).off=M(j).off;
  }
}

void TestFunctions::LocalLinearSwitch::map(doubleA& in,doubleA& out,bool randomInput,double noise){
  CHECK(M.N,"instantiate problem first");
  if(in.nd==2){ //batch case
    out.resize(in.d0,outDim());
    for(uint k=0;k<in.d0;k++) map(in[k](),out[k](),randomInput,noise);
    return;
  }
  CHECK(randomInput || in.N==inDim(),"wrong input dimension");
  
  //static MT::Parameter<double> outNoise("TestOutNoise"),switchNoise("TestSwitchNoise");
  
  uint i,m; double d,md;
  if(randomInput){ in.resize(inDim()); rndUni(in,-1,1); }
  if(!smooth){
    //find closest:
    m=0; md=sqrDistance(M(0).cen,in);
    for(i=1;i<M.N;i++){
      d=sqrDistance(M(i).cen,in);//+rnd.gauss(noise);
      if(d<md){ m=i; md=d; }
    }
    //map on output:
    out.resize(outDim());
    out=M(m).off + M(m).mat*in;
  }else{
    out.resize(outDim()); out.setZero();
    double w,W=0.;
    for(i=0;i<M.N;i++){
      w=exp(-.5*sqrDistance(M(i).cen,in)/M(i).sig);
      out+=w * (M(i).off + M(i).mat*in);
      W+=w;
    }
    out/=W;
  }
  if(noise) rndGauss(out,noise,true);
}

void TestFunctions::MAXSAT::readCNF(const char* file){
  std::ifstream fil(file);
  if(!fil.good()) HALT("error opening MAXSAT cnf file "<<file);
  fil >>(const char*)"c This Formular is generated by mcnf";
  fil >>(const char*)"c";
  fil >>(const char*)"c    horn? no";
  fil >>(const char*)"c    forced? no";
  fil >>(const char*)"c    mixed sat? no";
  fil >>(const char*)"c    clause length = 3";
  fil >>(const char*)"c";
  fil >>(const char*)"p cnf" >>vars >>clauses;
  if(!fil.good()) HALT("error when reading MAXSAT cnf file "<<file);
  
  uint i,j;
  fct.resize(clauses,3);
  fct=0;
  for(i=0;i<fct.d0;i++){
    for(j=0;j<fct.d1;j++){
      fil >>fct(i,j);
      CHECK((uint)abs(fct(i,j))<=vars,"readCNF: variable index "<<fct(i,j)<<" out of range");
    }
    fil >>(const char*)"0";
  }
}

double TestFunctions::MAXSAT::sat(const MT::Array<byte>& x,std::ostream *talk){
  CHECK(x.N==vars,"MAXSAT: sequence length "<<x.N<<" != MAXSAT length "<<vars);
  uint i,j,sat=0;
  bool S;
  if(talk){
    x.ioraw();
    (*talk) <<"sequence = <"; x.write(*talk); (*talk) <<std::endl;
  }
  for(i=0;i<fct.d0;i++){
    S=false; //a clause is an OR between the predicates: only one of them has to be true
    for(j=0;!S && j<fct.d1;j++){
      if(fct(i,j)==0) S=true; //0 is z wild card
      if(fct(i,j)>0 &&  (x( fct(i,j)-1) & 1)) S=true; //positive index needs 1 value (in [1,..,vars])
      if(fct(i,j)<0 && !(x(-fct(i,j)-1) & 1)) S=true; //negative index needs 0 value
      //we only consider the first bit of the byte!!!
    }
    if(S) sat++;
    if(talk){
      if(S) (*talk) <<"yes: clause " <<i <<": ";
      if(!S) (*talk) <<"no:  clause " <<i <<": ";
      fct[i].write(*talk); (*talk) <<std::endl;
    }
  }
  return ((double)sat)/fct.d0;
}

void TestFunctions::MAXSAT::write(std::ostream& os) const{
  uint i,j;
  for(i=0;i<fct.d0;i++){
    os <<"clause " <<i <<" = ";
    for(j=0;j<fct.d1;j++){
      os <<fct(i,j) <<' ';	  
    }
    os <<std::endl;
  }
}

